# Yii2-sender

Базовый сендер для http-клиентов на Yii2

# Установка

`composer require iwaydev/yii2-sender`

# Использование

Пример использования сендера в Yii2. Добавить его, как компонент в конфиг приложения
```
'components' => [
    'sender' => [
        'class' => Yii2Sender::class,
        'url' => 'https://api.url',
        'version' => 'v1',
    ],
],
```
## Кэширование

По дефолту кэширование не используется.

Чтобы установить кэшер нужно передать его инстанс в компонент.
Время жизни кэша по дефолту равно 3600 сек., меняется также - передачей значения в компонент.
```
'components' => [
    'sender' => [
        'class' => Yii2Sender::class,
        'url' => 'https://api.url',
        'version' => 'v1',
        'cacheComponent' => 'cache',
        'cacheLifetime' => 3600,
    ],
],
```
Чтобы использовать инициализированный кэшер нужно в цепочке вызова указать метод `caching()`,
если данный метод не указан, то запрос будет отправлен без кэширования.
```
$response = Yii::$app->sender->caching()->get('v1/action');
```
## Логирование

По дефолту логирование не ведется.

Чтобы начать логирование нужно передать категорию логгера в компопнент.

```
'components' => [
    'sender' => [
        'class' => Yii2Sender::class,
        'url' => 'https://api.url',
        'version' => 'v1',
        'cacheComponent' => 'cache',
        'cacheLifetime' => 3600,
        'logCategory' => 'sender',
    ],
],
```

В лог пишутся все данные запроса и контент из ответа.

```
[2021-02-24 10:24:26] dev.INFO: your-x-uuid Request. Method: POST  
[2021-02-24 10:24:26] dev.INFO: your-x-uuid Request. Url: http://api.service-name.local/v1/action  
[2021-02-24 10:24:26] dev.INFO: your-x-uuid Request. Options: {"form_params":[],"headers":{"X-UUID":"your-x-uuid","Authorization":"Bearer json.web.token"}} 
[2021-02-24 10:24:26] dev.INFO: your-x-uuid Response. Content: {"data":1} 
```

Если требуется изменить место или способ логирования нужно передать категорию другого логгера в метод `setLogger()`, добавив его в цепочку вызовов.

**Внимание!** Логгер, переданный в `setLogger()` будет заменен на дефолтный(тот который был передан в конструкторе) после отправки запроса.

```
$response = Yii::$app->sender->get('v1/action'); // Логирование в 'service_name'
$response = Yii::$app->sender->setLogger(logs('logger_a'))->get('v1/action'); // Логирование в 'logger_a'
$response = Yii::$app->sender->setLogger(logs('logger_b'))->get('v1/action'); // Логирование в 'logger_b'
$response = Yii::$app->sender->get('v1/action'); // Логирование снова в 'service_name'
```

## Установка заголовка X-UUID

По дефолту заголовок генерируется при создании сендера.

Если нужно использовать конкретный, то его передаем в конструктор.

```
'components' => [
    'sender' => [
        'class' => Yii2Sender::class,
        'url' => 'https://api.url',
        'version' => 'v1',
        'cacheComponent' => 'cache',
        'cacheLifetime' => 3600,
        'logCategory' => 'sender',
        'xUuid' => '123e4567-e89b-12d3-a456-4266554',
    ],
],
```

Можно менять по ходу использования вызвав метод `setXUuid()`.
Если метод вызван без передачи значения, то будет сгенерирован новый случайный заголовок.

```
Yii::$app->sender->setXUuid(); // Генерирует новый случайный заголовок X-UUID
Yii::$app->sender->setXUuid('хочу-это-заголовок'); // Устанавливает переданное значение как новый заголовок X-UUID
```

## Обработка ошибок от сервиса

По дефолту все исключения в ответе от сервиса, к которому был отправлен запрос, глушатся и возвращаются в ответе метода.

Если нужно кинуть исключение при ошибке на запрос, то в цепочке вызова нужно указать метод `withErrors()`.

```
$response = Yii::$app->sender->get('v1/action'); // В случае ошибки будет возвращен ответ от сервиса
$response = Yii::$app->sender->withErrors()->get('v1/action'); // В случае ошибки будет брошено исключение с текстом ошибки из ответа сервиса
```
